@@ -41,8 +41,7 @@ from AnyQt.QtCore import (
     QSettings, QStandardPaths, QEvent, QAbstractItemModel, QModelIndex,
 )
 from AnyQt.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
-
-import sip
+from AnyQt import sip
 
 from orangecanvas.utils import unique, name_lookup, markup, qualified_name
 from orangecanvas.utils.shtools import python_process, create_process
@@ -67,14 +66,12 @@ def normalize_name(name):
 
 
 def prettify_name(name):
-    if name.lower() == 'orange3-zh':
-        return '橙现智能'
     dash_split = name.split('-')
     # Orange3-ImageAnalytics => ImageAnalytics
-    if len(dash_split) > 1 and dash_split[0].lower() in ['orange', 'orange3']:
-        name = '-'.join(dash_split[1:])
+    orange_prefix = len(dash_split) > 1 and dash_split[0].lower() in ['orange', 'orange3']
+    name = ' '.join(dash_split[1:] if orange_prefix else dash_split)
     # ImageAnalytics => Image Analytics  # while keeping acronyms
-    return re.sub(r"(?<!^)([A-Z][a-z]|(?<=[a-z])[A-Z])", r" \1", name)
+    return re.sub(r"(?<!^)((?<![\s\d])[A-Z][a-z]|(?<=[a-z])[A-Z])", r" \1", name)
 
 
 class Installable(
@@ -333,7 +330,7 @@ class PluginsModel(QStandardItemModel):
     def __init__(self, parent=None, **kwargs):
         super().__init__(parent, **kwargs)
         self.setHorizontalHeaderLabels(
-            ["", self.tr("名称"), self.tr("版本"), self.tr("行动")]
+            ["", self.tr("名称"), self.tr("版本"), self.tr("操作")]
         )
 
     @staticmethod
@@ -361,7 +358,7 @@ class PluginsModel(QStandardItemModel):
         item1 = StateItem()
         item1.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable |
                        Qt.ItemIsUserCheckable |
-                       (Qt.ItemIsUserTristate if updatable else 0))
+                       (Qt.ItemIsUserTristate if updatable else Qt.NoItemFlags))
         item1.setEnabled(not (item_is_core and not updatable))
         item1.setData(item_is_core, HasConstraintRole)
 
@@ -545,7 +542,7 @@ class AddonManagerDialog(QDialog):
 
         self.__search = QLineEdit(
             objectName="filter-edit",
-            placeholderText=self.tr("过滤...")
+            placeholderText=self.tr("筛选...")
         )
         self.__addmore = QPushButton(
             self.tr("添加更多..."),
@@ -926,7 +923,7 @@ class AddonManagerDialog(QDialog):
             dlg.accept()
         buttons.accepted.connect(query)
         buttons.rejected.connect(dlg.reject)
-        dlg.exec_()
+        dlg.exec()
 
     @Slot(str, str)
     def __show_error_for_query(self, text, error_details):
@@ -1106,8 +1103,8 @@ class AddonManagerDialog(QDialog):
     def __on_installer_error(self, command, pkg, retcode, output):
         self.__on_installer_finished_common()
         message_error(
-            "运行子进程时出错", title="错误",
-            informative_text="{} 以非零状态退出。".format(command),
+            "An error occurred while running a subprocess", title="Error",
+            informative_text="{} exited with non zero status.".format(command),
             details="".join(output),
             parent=self
         )
@@ -1121,7 +1118,6 @@ class AddonManagerDialog(QDialog):
             icon = QMessageBox.Information
             buttons = QMessageBox.Ok | QMessageBox.Cancel
             title = '信息'
-            name = QApplication.applicationName() or 'Orange'
             text = ('{} 需要重新启动才能使更改生效。'
                     .format(name))
             msg_box = QMessageBox(icon, title, text, buttons, parent)
@@ -1129,7 +1125,7 @@ class AddonManagerDialog(QDialog):
             msg_box.setInformativeText('按 OK 键立即关闭 {} '
                                        .format(name))
             msg_box.button(QMessageBox.Cancel).setText('稍后关闭')
-            return msg_box.exec_()
+            return msg_box.exec()
 
         if QMessageBox.Ok == message_restart(self):
             self.accept()
@@ -1533,7 +1529,6 @@ class PipInstaller:
             cmd.append(package.package_url)
         run_command(cmd)
 
-
     def uninstall(self, dist):
         cmd = ["python", "-m", "pip", "uninstall", "--yes", dist.project_name]
         run_command(cmd)
@@ -1548,38 +1543,6 @@ class CondaInstaller:
         else:
             self.conda = False
 
-    # def _find_conda(self):
-    #     executable = sys.executable
-    #     bin = os.path.dirname(executable)
-    #
-    #     # posix
-    #     conda = os.path.join(bin, "conda")
-    #     if os.path.exists(conda):
-    #         return conda
-    #
-    #     # windows
-    #     conda = os.path.join(bin, "Scripts", "conda.bat")
-    #     if os.path.exists(conda):
-    #         # "activate" conda environment orange is running in
-    #         os.environ["CONDA_PREFIX"] = bin
-    #         os.environ["CONDA_DEFAULT_ENV"] = bin
-    #         return conda
-    #
-    # def install(self, pkg, raise_on_fail=False):
-    #     cmd = [self.conda, "install", "--yes", "--quiet",
-    #            self._normalize(pkg.name)]
-    #     run_command(cmd, raise_on_fail=raise_on_fail)
-    #
-    # def upgrade(self, pkg, raise_on_fail=False):
-    #     cmd = [self.conda, "upgrade", "--yes", "--quiet",
-    #            self._normalize(pkg.name)]
-    #     run_command(cmd, raise_on_fail=raise_on_fail)
-    #
-    # def uninstall(self, dist, raise_on_fail=False):
-    #     cmd = [self.conda, "uninstall", "--yes",
-    #            self._normalize(dist.project_name)]
-    #     run_command(cmd, raise_on_fail=raise_on_fail)
-    #
     # def _normalize(self, name):
     #     # Conda 4.3.30 is inconsistent, upgrade command is case sensitive
     #     # while install and uninstall are not. We assume that all conda
