@@ -3,6 +3,7 @@ Orange Canvas Main Window
 
 """
 import os
+import pkgutil
 import sys
 import logging
 import operator
@@ -18,21 +19,18 @@ from typing import (
     Tuple, TypeVar, Awaitable,
 )
 
-import pkg_resources
-
 from AnyQt.QtWidgets import (
     QMainWindow, QWidget, QAction, QActionGroup, QMenu, QMenuBar, QDialog,
     QFileDialog, QMessageBox, QVBoxLayout, QSizePolicy, QToolBar, QToolButton,
-    QDockWidget, QApplication, QShortcut, QPlainTextEdit,
-    QPlainTextDocumentLayout, QFileIconProvider
+    QDockWidget, QApplication, QShortcut, QFileIconProvider
 )
 from AnyQt.QtGui import (
-    QColor, QIcon, QDesktopServices, QKeySequence, QTextDocument,
+    QColor, QDesktopServices, QKeySequence,
     QWhatsThisClickedEvent, QShowEvent, QCloseEvent
 )
 from AnyQt.QtCore import (
     Qt, QObject, QEvent, QSize, QUrl, QFile, QByteArray, QFileInfo,
-    QSettings, QStandardPaths, QAbstractItemModel, QT_VERSION)
+    QSettings, QStandardPaths, QAbstractItemModel, QMimeData, QT_VERSION)
 
 try:
     from AnyQt.QtWebEngineWidgets import QWebEngineView
@@ -70,6 +68,7 @@ from .settings import UserSettingsDialog, category_state
 from ..document.schemeedit import SchemeEditWidget
 from ..document.quickmenu import QuickMenu
 from ..document.commands import UndoCommand
+from ..document import interactions
 from ..gui.itemmodels import FilterProxyModel
 from ..registry import WidgetRegistry, WidgetDescription, CategoryDescription
 from ..registry.qt import QtWidgetRegistry
@@ -77,41 +76,19 @@ from ..utils.settings import QSettings_readArray, QSettings_writeArray
 from ..utils.qinvoke import qinvoke
 from ..utils.pickle import Pickler, Unpickler, glob_scratch_swps, swp_name, \
     canvas_scratch_name_memo, register_loaded_swp
-from ..utils import unique, group_by_all, set_flag
+from ..utils import unique, group_by_all, set_flag, findf
 from ..utils.asyncutils import get_event_loop
 from ..utils.qobjref import qobjref
-
 from . import welcomedialog
 from . import addons
-
 from ..preview import previewdialog, previewmodel
-
 from .. import config
-
 from . import examples
+from ..resources import load_styled_svg_icon
 
 log = logging.getLogger(__name__)
 
 
-def resource_filename(path):
-    """
-    Return the resource filename path relative to the top level package.
-    """
-    return pkg_resources.resource_filename(config.__name__, path)
-
-
-def canvas_icons(name):
-    # type: (str) -> QIcon
-    """
-    Return the named canvas icon.
-    """
-    icon_file = QFile("canvas_icons:" + name)
-    if icon_file.exists():
-        return QIcon("canvas_icons:" + name)
-    else:
-        return QIcon(resource_filename(os.path.join("icons", name)))
-
-
 def user_documents_path():
     """
     Return the users 'Documents' folder path.
@@ -151,7 +128,7 @@ class DockWidget(QDockWidget):
 
 
 class CanvasMainWindow(QMainWindow):
-    SETTINGS_VERSION = 2
+    SETTINGS_VERSION = 3
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -164,7 +141,6 @@ class CanvasMainWindow(QMainWindow):
         self.__registry_model = None  # type: Optional[QAbstractItemModel]
         # Proxy widget registry model
         self.__proxy_model = None  # type: Optional[FilterProxyModel]
-        self.__streams = []  # type: List[TextStream]
 
         # TODO: Help view and manager to separate singleton instance.
         self.help = None  # type: HelpManager
@@ -197,9 +173,6 @@ class CanvasMainWindow(QMainWindow):
     def setup_ui(self):
         """Setup main canvas ui
         """
-
-        log.info("Setting up Canvas main window.")
-
         # Two dummy tool bars to reserve space
         self.__dummy_top_toolbar = FakeToolBar(
             objectName="__dummy_top_toolbar")
@@ -223,6 +196,7 @@ class CanvasMainWindow(QMainWindow):
         w.layout().setContentsMargins(20, 0, 10, 0)
 
         self.scheme_widget = SchemeEditWidget()
+        self.scheme_widget.setDropHandlers([interactions.PluginDropHandler(),])
         self.set_scheme(config.workflow_constructor(parent=self))
 
         # Save crash recovery swap file on changes to workflow
@@ -231,7 +205,7 @@ class CanvasMainWindow(QMainWindow):
         dropfilter = UrlDropEventFilter(self)
         dropfilter.urlDropped.connect(self.open_scheme_file)
         self.scheme_widget.setAcceptDrops(True)
-        self.scheme_widget.installEventFilter(dropfilter)
+        self.scheme_widget.view().viewport().installEventFilter(dropfilter)
 
         w.layout().addWidget(self.scheme_widget)
 
@@ -243,8 +217,9 @@ class CanvasMainWindow(QMainWindow):
         frame.setWidget(self.scheme_widget)
 
         # Window 'title'
+        self.__update_window_title()
         self.setWindowFilePath(self.scheme_widget.path())
-        self.scheme_widget.pathChanged.connect(self.setWindowFilePath)
+        self.scheme_widget.pathChanged.connect(self.__update_window_title)
         self.scheme_widget.modificationChanged.connect(self.setWindowModified)
 
         # QMainWindow's Dock widget
@@ -280,7 +255,7 @@ class CanvasMainWindow(QMainWindow):
         )
 
         self.dock_help = canvas_tool_dock.help
-        # self.dock_help.setMaximumHeight(150)
+        self.dock_help.setMaximumHeight(150)
         self.dock_help.document().setDefaultStyleSheet("h3, a {color: orange;}")
 
         self.dock_help.setDefaultText(
@@ -292,7 +267,7 @@ class CanvasMainWindow(QMainWindow):
         )
         self.dock_help_action = canvas_tool_dock.toggleQuickHelpAction()
         self.dock_help_action.setText(self.tr("显示帮助"))
-        self.dock_help_action.setIcon(canvas_icons("Info.svg"))
+        self.dock_help_action.setIcon(load_styled_svg_icon("Info.svg", self.canvas_toolbar))
 
         self.canvas_tool_dock = canvas_tool_dock
 
@@ -316,9 +291,11 @@ class CanvasMainWindow(QMainWindow):
          self.canvas_align_to_grid_action,
          self.canvas_text_action, self.canvas_arrow_action,) = tool_actions
 
-        self.canvas_align_to_grid_action.setIcon(canvas_icons("Grid.svg"))
-        self.canvas_text_action.setIcon(canvas_icons("Text Size.svg"))
-        self.canvas_arrow_action.setIcon(canvas_icons("Arrow.svg"))
+        self.canvas_align_to_grid_action.setIcon(load_styled_svg_icon("Grid.svg", self.canvas_toolbar))
+        self.canvas_text_action.setIcon(load_styled_svg_icon("Text Size.svg", self.canvas_toolbar))
+        self.canvas_arrow_action.setIcon(load_styled_svg_icon("Arrow.svg", self.canvas_toolbar))
+        self.freeze_action.setIcon(load_styled_svg_icon('Pause.svg', self.canvas_toolbar))
+        self.show_properties_action.setIcon(load_styled_svg_icon("Document Info.svg", self.canvas_toolbar))
 
         dock_actions = [
             self.show_properties_action,
@@ -374,8 +351,9 @@ class CanvasMainWindow(QMainWindow):
 
         self.help_dock = DockWidget(
             self.tr("Help"), self, objectName="help-dock",
-            allowedAreas=Qt.RightDockWidgetArea | Qt.BottomDockWidgetArea,
-            visible=False
+            allowedAreas=Qt.NoDockWidgetArea,
+            visible=False,
+            floating=True,
         )
         if QWebEngineView is not None:
             self.help_view = QWebEngineView()
@@ -391,8 +369,6 @@ class CanvasMainWindow(QMainWindow):
             manager.setCache(cache)
 
         self.help_dock.setWidget(self.help_view)
-        self.addDockWidget(Qt.RightDockWidgetArea, self.help_dock)
-
         self.setMinimumSize(600, 500)
 
     def setup_actions(self):
@@ -404,7 +380,7 @@ class CanvasMainWindow(QMainWindow):
             toolTip=self.tr("打开新工作流。"),
             triggered=self.new_workflow_window,
             shortcut=QKeySequence.New,
-            icon=canvas_icons("New.svg")
+            icon=load_styled_svg_icon("New.svg")
         )
         self.open_action = QAction(
             self.tr("打开"), self,
@@ -412,7 +388,7 @@ class CanvasMainWindow(QMainWindow):
             toolTip=self.tr("打开工作流。"),
             triggered=self.open_scheme,
             shortcut=QKeySequence.Open,
-            icon=canvas_icons("Open.svg")
+            icon=load_styled_svg_icon("Open.svg")
         )
         self.open_and_freeze_action = QAction(
             self.tr("打开并冻结"), self,
@@ -421,7 +397,7 @@ class CanvasMainWindow(QMainWindow):
             triggered=self.open_and_freeze_scheme
         )
         self.open_and_freeze_action.setShortcut(
-            QKeySequence(Qt.ControlModifier | Qt.AltModifier | Qt.Key_O)
+            QKeySequence("Ctrl+Alt+O")
         )
         self.close_window_action = QAction(
             self.tr("关闭窗口"), self,
@@ -480,7 +456,7 @@ class CanvasMainWindow(QMainWindow):
             self.tr("开始"), self,
             objectName="get-started-action",
             toolTip=self.tr("查看“入门”介绍。"),
-            icon=canvas_icons("Documentation.svg")
+            icon=load_styled_svg_icon("Documentation.svg")
         )
         config_url_action(self.get_started_action, "Quick Start")
 
@@ -488,7 +464,7 @@ class CanvasMainWindow(QMainWindow):
             self.tr("视频教程"), self,
             objectName="screencasts-action",
             toolTip=self.tr("查看视频教程"),
-            icon=canvas_icons("YouTube.svg"),
+            icon=load_styled_svg_icon("YouTube.svg"),
         )
         config_url_action(self.get_started_screencasts_action, "Screencasts")
 
@@ -496,7 +472,7 @@ class CanvasMainWindow(QMainWindow):
             self.tr("文档"), self,
             objectName="documentation-action",
             toolTip=self.tr("查看参考文档。"),
-            icon=canvas_icons("Documentation.svg"),
+            icon=load_styled_svg_icon("Documentation.svg"),
         )
         config_url_action(self.documentation_action, "Documentation")
 
@@ -505,7 +481,7 @@ class CanvasMainWindow(QMainWindow):
             objectName="examples-action",
             toolTip=self.tr("浏览示例工作流。"),
             triggered=self.examples_dialog,
-            icon=canvas_icons("Examples.svg")
+            icon=load_styled_svg_icon("Examples.svg")
         )
 
         self.about_action = QAction(
@@ -527,16 +503,15 @@ class CanvasMainWindow(QMainWindow):
             objectName="recent-action",
             toolTip=self.tr("浏览并打开最近的工作流。"),
             triggered=self.recent_scheme,
-            shortcut=QKeySequence(Qt.ControlModifier |
-                                  (Qt.ShiftModifier | Qt.Key_R)),
-            icon=canvas_icons("Recent.svg")
+            shortcut=QKeySequence("Ctrl+Shift+R"),
+            icon=load_styled_svg_icon("Recent.svg")
         )
         self.reload_last_action = QAction(
             self.tr("重新加载上一个工作流"), self,
             objectName="reload-last-action",
             toolTip=self.tr("重新加载上次打开的工作流。"),
             triggered=self.reload_last,
-            shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_R)
+            shortcut=QKeySequence("Ctrl+R")
         )
         self.clear_recent_action = QAction(
             self.tr("清除菜单"), self,
@@ -549,8 +524,8 @@ class CanvasMainWindow(QMainWindow):
             objectName="show-properties-action",
             toolTip=self.tr("显示工作流属性。"),
             triggered=self.show_scheme_properties,
-            shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_I),
-            icon=canvas_icons("Document Info.svg")
+            shortcut=QKeySequence("Ctrl+I"),
+            icon=load_styled_svg_icon("Document Info.svg")
         )
 
         self.canvas_settings_action = QAction(
@@ -579,7 +554,7 @@ class CanvasMainWindow(QMainWindow):
             self.minimize_action = QAction(
                 self.tr("缩小"), self,
                 triggered=self.showMinimized,
-                shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_M)
+                shortcut=QKeySequence("Ctrl+M")
             )
             self.zoom_action = QAction(
                 self.tr("放大"), self,
@@ -593,15 +568,14 @@ class CanvasMainWindow(QMainWindow):
             checkable=True,
             toolTip=self.tr("冻结信号传播 (Shift+F)"),
             toggled=self.set_signal_freeze,
-            icon=canvas_icons("Pause.svg")
+            icon=load_styled_svg_icon("Pause.svg")
         )
 
         self.toggle_tool_dock_expand = QAction(
             self.tr("展开工具底座"), self,
             objectName="toggle-tool-dock-expand",
             checkable=True,
-            shortcut=QKeySequence(Qt.ControlModifier |
-                                  (Qt.ShiftModifier | Qt.Key_D)),
+            shortcut=QKeySequence("Ctrl+Shift+D"),
             triggered=self.set_tool_dock_expanded
         )
         self.toggle_tool_dock_expand.setChecked(True)
@@ -814,6 +788,15 @@ class CanvasMainWindow(QMainWindow):
 
         self.__update_from_settings()
 
+    def __update_window_title(self):
+        path = self.current_document().path()
+        if path:
+            self.setWindowTitle("")
+            self.setWindowFilePath(path)
+        else:
+            self.setWindowFilePath("")
+            self.setWindowTitle(self.tr("Untitled [*]"))
+
     def setWindowFilePath(self, filePath):  # type: (str) -> None
         if sys.platform == "darwin":
             super().setWindowFilePath(filePath)
@@ -905,6 +888,7 @@ class CanvasMainWindow(QMainWindow):
                 statistics = scheme_widget.usageStatistics()
                 statistics.begin_action(UsageStatistics.ToolboxClick)
                 scheme_widget.createNewNode(widget_desc)
+                scheme_widget.view().setFocus(Qt.OtherFocusReason)
 
     def on_quick_category_action(self, action):
         """The quick category menu action triggered.
@@ -926,9 +910,10 @@ class CanvasMainWindow(QMainWindow):
             if i != -1:
                 popup.setModel(model)
                 popup.setRootIndex(model.index(i, 0))
+                popup.adjustSize()
                 button = self.quick_category.buttonForAction(action)
                 pos = popup_position_from_source(popup, button)
-                action = popup.exec_(pos)
+                action = popup.exec(pos)
                 if action is not None:
                     self.on_tool_box_widget_activated(action)
 
@@ -1010,34 +995,31 @@ class CanvasMainWindow(QMainWindow):
         window.setAttribute(Qt.WA_DeleteOnClose)
         window.setGeometry(self.geometry().translated(20, 20))
         window.setStyleSheet(self.styleSheet())
+        window.setWindowIcon(self.windowIcon())
         if self.widget_registry is not None:
             window.set_widget_registry(self.widget_registry)
         window.restoreState(self.saveState(self.SETTINGS_VERSION),
                             self.SETTINGS_VERSION)
         window.set_tool_dock_expanded(self.dock_widget.expanded())
         window.set_float_widgets_on_top_enabled(self.float_widgets_on_top_action.isChecked())
-        logview = window.output_view()  # type: OutputView
 
-        te = logview.findChild(QPlainTextEdit)
+        output = window.output_view()  # type: OutputView
+        doc = self.output_view().document()
+        doc = doc.clone(output)
+        output.setDocument(doc)
 
-        doc = self.output_view().findChild(QPlainTextEdit).document()
-        # first clone the existing document and set it on the new instance
-        doc = doc.clone(parent=te)  # type: QTextDocument
-        doc.setDocumentLayout(QPlainTextDocumentLayout(doc))
-        te.setDocument(doc)
+        def is_connected(stream: TextStream) -> bool:
+            item = findf(doc.connectedStreams(), lambda s: s is stream)
+            return item is not None
 
-        # route the stdout/err if possible
+        # # route the stdout/err if possible
+        # TODO: Deprecate and remove this behaviour (use connectStream)
         stdout, stderr = sys.stdout, sys.stderr
-        if isinstance(stdout, TextStream):
-            stdout.stream.connect(logview.write)
+        if isinstance(stdout, TextStream) and not is_connected(stdout):
+            doc.connectStream(stdout)
 
-        if isinstance(stderr, TextStream):
-            err_formatter = logview.formatted(color=Qt.red)
-            stderr.stream.connect(err_formatter.write)
-
-        for stream in self.__streams:
-            if stream not in (stdout, stderr):
-                window.connect_output_stream(stream)
+        if isinstance(stderr, TextStream) and not is_connected(stderr):
+            doc.connectStream(stderr, color=Qt.red)
 
         CanvasMainWindow._instances.append(window)
         window.destroyed.connect(
@@ -1161,6 +1143,20 @@ class CanvasMainWindow(QMainWindow):
                     return
                 f.seek(0, os.SEEK_SET)
                 new_scheme = self.new_scheme_from_contents_and_path(f, filename)
+        except readwrite.UnsupportedFormatVersionError:
+            mb = QMessageBox(
+                self, windowTitle=self.tr("Error"),
+                icon=QMessageBox.Critical,
+                text=self.tr("Unsupported format version"),
+                informativeText=self.tr(
+                    "The file was saved in a format not supported by this "
+                    "application."
+                ),
+                detailedText="".join(traceback.format_exc()),
+            )
+            mb.setAttribute(Qt.WA_DeleteOnClose)
+            mb.setWindowModality(Qt.WindowModal)
+            mb.open()
         except Exception as err:
             mb = QMessageBox(
                 parent=self, windowTitle=self.tr("Error"),
@@ -1237,7 +1233,8 @@ class CanvasMainWindow(QMainWindow):
         workflow: Optional[Scheme]
         """
         new_scheme = config.workflow_constructor(parent=self)
-        new_scheme.set_runtime_env("basedir", os.path.dirname(path))
+        new_scheme.set_runtime_env(
+            "basedir", os.path.abspath(os.path.dirname(path)))
         errors = []  # type: List[Exception]
         try:
             new_scheme.load_from(
@@ -1259,7 +1256,7 @@ class CanvasMainWindow(QMainWindow):
                 self.tr("加载工作流时出错。"),
                 title=self.tr("只加载部分工作流"),
                 informative_text=self.tr(
-                     "在工作流中加载某些小部件/链接时出现问题。"
+                     "在工作流中加载某些小部件/链接时出现问题."
                 ),
                 details=details,
                 parent=self,
@@ -1417,7 +1414,7 @@ class CanvasMainWindow(QMainWindow):
         selected = message_question(
             message,
             self.tr("保存更改？"),
-            self.tr("如果不保存更改，更改将丢失。"),
+            self.tr("如果不保存更改，更改将丢失."),
             buttons=QMessageBox.Save | QMessageBox.Cancel | \
                     QMessageBox.Discard,
             default_button=QMessageBox.Save,
@@ -1509,7 +1506,7 @@ class CanvasMainWindow(QMainWindow):
         # existing scheme file if `scheme.save_to` raises an error.
         buffer = io.BytesIO()
         try:
-            scheme.set_runtime_env("basedir", dirname)
+            scheme.set_runtime_env("basedir", os.path.abspath(dirname))
             scheme.save_to(buffer, pretty=True, pickle_fallback=True)
         except Exception:
             log.error("将 %r 保存到 %r 时出错", scheme, filename, exc_info=True)
@@ -1527,12 +1524,13 @@ class CanvasMainWindow(QMainWindow):
             self.clear_swp()
             return True
         except FileNotFoundError as ex:
-            log.error("%s 正在保存 '%s'", type(ex).__name__, filename,
+            log.error("%s saving '%s'", type(ex).__name__, filename,
                       exc_info=True)
             message_warning(
-                self.tr('无法保存工作流程 "%s"。该路径不存在') % title,
+                self.tr('Workflow "%s" could not be saved. The path does '
+                        'not exist') % title,
                 title="",
-                informative_text=self.tr("选择其他位置"),
+                informative_text=self.tr("Choose another location."),
                 parent=self
             )
             return False
@@ -1540,10 +1538,10 @@ class CanvasMainWindow(QMainWindow):
             log.error("%s 正在保存 '%s'", type(ex).__name__, filename,
                       exc_info=True)
             message_warning(
-                self.tr('无法保存工作流 "%s"。您没有写入权限。') % title,
+                self.tr('工作流 "%s" 无法保存. 没有写入权限.') % title,
                 title="",
                 informative_text=self.tr(
-                    "更改文件系统权限或选择其他位置。"),
+                    "更改系统权限或选择其他位置."),
                 parent=self
             )
             return False
@@ -1551,7 +1549,7 @@ class CanvasMainWindow(QMainWindow):
             log.error("%s 正在保存 '%s'", type(ex).__name__, filename,
                       exc_info=True)
             message_warning(
-                self.tr('无法保存工作流 "%s"。') % title,
+                self.tr('无法保存工作流 "%s.') % title,
                 title="",
                 informative_text=os.strerror(ex.errno),
                 exc_info=True,
@@ -1796,7 +1794,7 @@ class CanvasMainWindow(QMainWindow):
 
         model.delayedScanUpdate()
 
-        status = dialog.exec_()
+        status = dialog.exec()
 
         index = dialog.currentIndex()
 
@@ -1820,7 +1818,7 @@ class CanvasMainWindow(QMainWindow):
         items = [previewmodel.PreviewItem(path=t.abspath()) for t in tutors]
         dialog = previewdialog.PreviewDialog(self)
         model = previewmodel.PreviewModel(dialog, items=items)
-        title = self.tr("示例工作流")
+        title = self.tr("Example Workflows")
         dialog.setWindowTitle(title)
         template = ('<h3 style="font-size: 26px">\n'
                     '{0}\n'
@@ -1830,7 +1828,7 @@ class CanvasMainWindow(QMainWindow):
         dialog.setModel(model)
 
         model.delayedScanUpdate()
-        status = dialog.exec_()
+        status = dialog.exec()
         index = dialog.currentIndex()
 
         dialog.deleteLater()
@@ -1874,13 +1872,12 @@ class CanvasMainWindow(QMainWindow):
         def browse_examples():
             if self.examples_dialog() == QDialog.Accepted:
                 dialog.accept()
-
         new_action = QAction(
             self.tr("新建"), dialog,
             toolTip=self.tr("打开新工作流。"),
             triggered=new_scheme,
             shortcut=QKeySequence.New,
-            icon=canvas_icons("New.svg")
+            icon=load_styled_svg_icon("New.svg")
         )
 
         open_action = QAction(
@@ -1889,7 +1886,7 @@ class CanvasMainWindow(QMainWindow):
             toolTip=self.tr("打开工作流。"),
             triggered=open_scheme,
             shortcut=QKeySequence.Open,
-            icon=canvas_icons("Open.svg")
+            icon=load_styled_svg_icon("Open.svg")
         )
 
         recent_action = QAction(
@@ -1897,9 +1894,8 @@ class CanvasMainWindow(QMainWindow):
             objectName="welcome-recent-action",
             toolTip=self.tr("浏览并打开最近的工作流。"),
             triggered=open_recent,
-            shortcut=QKeySequence(Qt.ControlModifier |
-                                  (Qt.ShiftModifier | Qt.Key_R)),
-            icon=canvas_icons("Recent.svg")
+            shortcut=QKeySequence("Ctrl+Shift+R"),
+            icon=load_styled_svg_icon("Recent.svg")
         )
 
         examples_action = QAction(
@@ -1907,7 +1903,7 @@ class CanvasMainWindow(QMainWindow):
             objectName="welcome-examples-action",
             toolTip=self.tr("浏览示例工作流。"),
             triggered=browse_examples,
-            icon=canvas_icons("Examples.svg")
+            icon=load_styled_svg_icon("Examples.svg")
         )
 
         bottom_row = [self.get_started_action, examples_action,
@@ -1927,7 +1923,7 @@ class CanvasMainWindow(QMainWindow):
             settings.value("startup/show-welcome-screen", True, type=bool)
         )
 
-        status = dialog.exec_()
+        status = dialog.exec()
 
         settings.setValue("startup/show-welcome-screen",
                           dialog.showAtStartup())
@@ -1965,7 +1961,7 @@ class CanvasMainWindow(QMainWindow):
         dlg = self.scheme_properties_dialog()
         dlg.setAutoCommit(False)
         dlg.setScheme(scheme)
-        status = dlg.exec_()
+        status = dlg.exec()
 
         if status == QDialog.Accepted:
             editor = dlg.editor
@@ -2023,7 +2019,7 @@ class CanvasMainWindow(QMainWindow):
         dlg = UserSettingsDialog(self)
         dlg.setWindowTitle(self.tr("偏好"))
         dlg.show()
-        status = dlg.exec_()
+        status = dlg.exec()
 
         if status == 0:
             self.user_preferences_changed_notify_all()
@@ -2051,14 +2047,14 @@ class CanvasMainWindow(QMainWindow):
                         "安装加载项的权限不足。 尝试启动 {name} 作为系统管理员或在用户文件夹中安装 {name} "
                         "as a system administrator or install {name} in user folders."
                         .format(name=name),
-                        parent=self).exec_()
+                        parent=self).exec()
         dlg = addons.AddonManagerDialog(
             self, windowTitle=self.tr("插件"), modal=True
         )
         dlg.setStyle(QApplication.style())
         dlg.setAttribute(Qt.WA_DeleteOnClose)
         dlg.start(config.default)
-        return dlg.exec_()
+        return dlg.exec()
 
     def set_float_widgets_on_top_enabled(self, enabled):
         # type: (bool) -> None
@@ -2081,7 +2077,7 @@ class CanvasMainWindow(QMainWindow):
         """
         dlg = AboutDialog(self)
         dlg.setAttribute(Qt.WA_DeleteOnClose)
-        dlg.exec_()
+        dlg.exec()
 
     def add_recent_scheme(self, title, path):
         # type: (str, str) -> None
@@ -2274,7 +2270,7 @@ class CanvasMainWindow(QMainWindow):
                     self.restoreGeometry(geom_data)
 
             state = settings.value("state", QByteArray(), type=QByteArray)
-            # Restore dock/toolbar state is not already done so
+            # Restore dock/toolbar state if not already done so
             if state and not self.__did_restore:
                 self.restoreState(state, version=self.SETTINGS_VERSION)
 
@@ -2328,8 +2324,6 @@ class CanvasMainWindow(QMainWindow):
                 action.trigger()
             else:
                 log.warning("No target action found for %r", url.toString())
-        elif url.scheme() == 'https' or url.scheme() == 'http':
-            QDesktopServices.openUrl(QUrl(url))
 
     def event(self, event):
         # type: (QEvent) -> bool
@@ -2341,7 +2335,6 @@ class CanvasMainWindow(QMainWindow):
         elif event.type() == QEvent.WhatsThisClicked:
             event = cast(QWhatsThisClickedEvent, event)
             self.whatsThisClickedEvent(event)
-
             return True
         return super().event(event)
 
@@ -2458,7 +2451,14 @@ class CanvasMainWindow(QMainWindow):
                                             defaultValue=True,
                                             type=bool)
         self.scheme_widget.setChannelNamesVisible(show_channel_names)
-
+        open_anchors_ = settings.value(
+            "open-anchors-on-hover", defaultValue=False, type=bool
+        )
+        if open_anchors_:
+            open_anchors = SchemeEditWidget.OpenAnchors.Always
+        else:
+            open_anchors = SchemeEditWidget.OpenAnchors.OnShift
+        self.scheme_widget.setOpenAnchorsMode(open_anchors)
         node_animations = settings.value("enable-node-animations",
                                          defaultValue=False,
                                          type=bool)
@@ -2491,18 +2491,16 @@ class CanvasMainWindow(QMainWindow):
         The `stream` will be 'inherited' by new windows created by
         `create_new_window`.
         """
-        self.__streams.append(stream)
-        view = self.output_view()
-        stream.stream.connect(view.write)
+        doc = self.output_view().document()
+        doc.connectStream(stream)
 
     def disconnect_output_stream(self, stream: TextStream):
         """
         Disconnect a :class:`TextStream` instance from this window's
         output view.
         """
-        self.__streams.remove(stream)
-        view = self.output_view()
-        stream.stream.disconnect(view.write)
+        doc = self.output_view().document()
+        doc.disconnectStream(stream)
 
 
 def updated_flags(flags, mask, state):
@@ -2527,7 +2525,7 @@ def index(sequence, *what, **kwargs):
         item_key = key(item)
         if predicate(what, item_key):
             return i
-    raise ValueError("%r 不按顺序排列" % what)
+    raise ValueError("%r not in sequence" % what)
 
 
 def category_filter_function(state):
@@ -2543,27 +2541,29 @@ def category_filter_function(state):
 class UrlDropEventFilter(QObject):
     urlDropped = Signal(QUrl)
 
+    def acceptsDrop(self, mime: QMimeData) -> bool:
+        if mime.hasUrls() and len(mime.urls()) == 1:
+            url = mime.urls()[0]
+            if url.scheme() == "file":
+                filename = url.toLocalFile()
+                _, ext = os.path.splitext(filename)
+                if ext == ".ows":
+                    return True
+        return False
+
     def eventFilter(self, obj, event):
         etype = event.type()
         if etype == QEvent.DragEnter or etype == QEvent.DragMove:
-            mime = event.mimeData()
-            if mime.hasUrls() and len(mime.urls()) == 1:
-                url = mime.urls()[0]
-                if url.scheme() == "file":
-                    filename = url.toLocalFile()
-                    _, ext = os.path.splitext(filename)
-                    if ext == ".ows":
-                        event.acceptProposedAction()
-                        return True
-
-        elif etype == QEvent.Drop:
-            mime = event.mimeData()
-            urls = mime.urls()
-            if urls:
-                url = urls[0]
-                self.urlDropped.emit(url)
+            if self.acceptsDrop(event.mimeData()):
+                event.acceptProposedAction()
                 return True
-
+        elif etype == QEvent.Drop:
+            if self.acceptsDrop(event.mimeData()):
+                urls = event.mimeData().urls()
+                if urls:
+                    url = urls[0]
+                    self.urlDropped.emit(url)
+                    return True
         return super().eventFilter(obj, event)
 
 
@@ -2585,15 +2585,7 @@ def scheme_requires(
     desc = readwrite.parse_ows_stream(stream)
     if registry is not None:
         desc = readwrite.resolve_replaced(desc, registry)
-        projects = []
-        for m in desc.nodes:
-            if m.project_name:
-                if m.project_name == 'Orange3':
-                    projects.append('Orange3-zh')
-                else:
-                    projects.append(m.project_name)
-        return list(unique(projects))
-    # return list(unique(m.project_name for m in desc.nodes if m.project_name))
+    return list(unique(m.project_name for m in desc.nodes if m.project_name))
 
 
 K = TypeVar("K")
