@@ -5,6 +5,7 @@ User settings/preference dialog
 """
 import sys
 import logging
+import warnings
 from functools import cmp_to_key
 from collections import namedtuple
 
@@ -282,10 +283,19 @@ class UserSettingsDialog(QMainWindow):
             objectName="enable-node-animations",
             toolTip=self.tr("为工作流中的节点启用阴影和ping动画。")
         )
+        cb_anchors = QCheckBox(
+            self.tr("Open anchors on hover"),
+            objectName="open-anchors-on-hover",
+            toolTip=self.tr(
+                "Open/expand node anchors on mouse hover (if unchecked the "
+                "anchors are expanded when Shift key is pressed)."
+            ),
+        )
         self.bind(cb_anim, "checked", "schemeedit/enable-node-animations")
+        self.bind(cb_anchors, "checked", "schemeedit/open-anchors-on-hover")
         nodes.layout().addWidget(cb_anim)
-
-        form.addRow(self.tr("结点"), nodes)
+        nodes.layout().addWidget(cb_anchors)
+        form.addRow(self.tr("节点"), nodes)
 
         links = QWidget(self, objectName="links")
         links.setLayout(QVBoxLayout())
@@ -394,6 +404,7 @@ class UserSettingsDialog(QMainWindow):
                         self.tr("调试")])
         self.bind(combo, "currentIndex", "logging/level")
         form.addRow(self.tr("记录"), combo)
+
         box = QWidget()
         layout = QVBoxLayout()
         layout.setContentsMargins(0, 0, 0, 0)
@@ -546,14 +557,20 @@ class UserSettingsDialog(QMainWindow):
                 log.error("Error reseting %r", source.propertyName,
                           exc_info=True)
 
-    def exec_(self):
+    def exec(self):
         self.__loop = QEventLoop()
         self.show()
-        status = self.__loop.exec_()
+        status = self.__loop.exec()
         self.__loop = None
         refresh_proxies()
         return status
 
+    def exec_(self, *args, **kwargs):
+        warnings.warn(
+            "exec_ is deprecated, use exec", DeprecationWarning, stacklevel=2
+        )
+        return self.exec(*args, **kwargs)
+
     def hideEvent(self, event):
         super().hideEvent(event)
         if self.__loop is not None:
