@@ -46,7 +46,7 @@ class EditLinksDialog(QDialog):
     >>> dlg.setNodes(source_node, sink_node)
     >>> dlg.setLinks([(source_node.output_channel("Data"),
     ...                sink_node.input_channel("Data"))])
-    >>> if dlg.exec_() == EditLinksDialog.Accepted:
+    >>> if dlg.exec() == EditLinksDialog.Accepted:
     ...     new_links = dlg.links()
     ...
     """
@@ -75,12 +75,9 @@ class EditLinksDialog(QDialog):
                                    QDialogButtonBox.Cancel |
                                    QDialogButtonBox.Reset,
                                    Qt.Horizontal)
-        
-        buttons.button(QDialogButtonBox.Ok).setText('确认')
-        buttons.button(QDialogButtonBox.Cancel).setText('取消')
 
         clear_button = buttons.button(QDialogButtonBox.Reset)
-        clear_button.setText(self.tr("清除所有"))
+        clear_button.setText(self.tr("Clear All"))
 
         buttons.accepted.connect(self.accept)
         buttons.rejected.connect(self.reject)
@@ -125,9 +122,11 @@ class EditLinksDialog(QDialog):
 
     def __onGeometryChanged(self):
         size = self.scene.editWidget.size()
-        left, top, right, bottom = self.getContentsMargins()
-        self.view.setFixedSize(size.toSize() + \
-                               QSize(left + right + 4, top + bottom + 4))
+        m = self.contentsMargins()
+        self.view.setFixedSize(
+            size.toSize() + QSize(m.left() + m.right() + 4,
+                                  m.top() + m.bottom() + 4)
+        )
         self.view.setSceneRect(self.scene.editWidget.geometry())
 
 
@@ -299,7 +298,7 @@ class LinksEditWidget(QGraphicsWidget):
                 eventPos = event.pos()
                 line.setLine(start.x(), start.y(), eventPos.x(), eventPos.y())
 
-                pen = QPen(self.palette().color(QPalette.Foreground), 4)
+                pen = QPen(self.palette().color(QPalette.WindowText), 4)
                 pen.setCapStyle(Qt.RoundCap)
                 line.setPen(pen)
                 line.show()
