@@ -13,30 +13,33 @@ All interactions are subclasses of :class:`UserInteraction`.
 
 """
 import typing
-from typing import Optional, Any, Tuple, List, Set, Iterable
+from typing import Optional, Any, Tuple, List, Set, Iterable, Sequence, Dict
 
+import abc
 import logging
 from functools import reduce
 
-
-
 from AnyQt.QtWidgets import (
-    QApplication, QGraphicsRectItem, QUndoCommand, QGraphicsSceneMouseEvent,
+    QApplication, QGraphicsRectItem, QGraphicsSceneMouseEvent,
     QGraphicsSceneContextMenuEvent, QWidget, QGraphicsItem,
+    QGraphicsSceneDragDropEvent, QMenu, QAction
 )
 from AnyQt.QtGui import QPen, QBrush, QColor, QFontMetrics, QKeyEvent, QFont
 from AnyQt.QtCore import (
     Qt, QObject, QCoreApplication, QSizeF, QPointF, QRect, QRectF, QLineF,
-    QPoint,
+    QPoint, QMimeData,
 )
 from AnyQt.QtCore import pyqtSignal as Signal
 
 from orangecanvas.document.commands import UndoCommand
 from .usagestatistics import UsageStatistics
 from ..registry.description import WidgetDescription, OutputSignal, InputSignal
-from ..registry.qt import QtWidgetRegistry
+from ..registry.qt import QtWidgetRegistry, tooltip_helper, whats_this_helper
 from .. import scheme
-from ..scheme import SchemeNode as Node, SchemeLink as Link, Scheme, compatible_channels
+from ..scheme import (
+    SchemeNode as Node, SchemeLink as Link, Scheme, WorkflowEvent,
+    compatible_channels
+)
 from ..canvas import items
 from ..canvas.items import controlpoints
 from ..gui.quickhelp import QuickHelpTipEvent
@@ -50,6 +53,12 @@ if typing.TYPE_CHECKING:
     #: Output/Input pair of a link
     OIPair = Tuple[OutputSignal, InputSignal]
 
+try:
+    from importlib.metadata import EntryPoint, entry_points
+except ImportError:
+    from importlib_metadata import EntryPoint, entry_points
+
+
 log = logging.getLogger(__name__)
 
 
@@ -235,6 +244,42 @@ class UserInteraction(QObject):
         """
         return False
 
+    def dragEnterEvent(self, event):
+        # type: (QGraphicsSceneDragDropEvent) -> bool
+        """
+        Handle a `QGraphicsScene.dragEnterEvent`
+
+        .. versionadded:: 0.1.20
+        """
+        return False
+
+    def dragMoveEvent(self, event):
+        # type: (QGraphicsSceneDragDropEvent) -> bool
+        """
+        Handle a `QGraphicsScene.dragMoveEvent`
+
+        .. versionadded:: 0.1.20
+        """
+        return False
+
+    def dragLeaveEvent(self, event):
+        # type: (QGraphicsSceneDragDropEvent) -> bool
+        """
+        Handle a `QGraphicsScene.dragLeaveEvent`
+
+        .. versionadded:: 0.1.20
+        """
+        return False
+
+    def dropEvent(self, event):
+        # type: (QGraphicsSceneDragDropEvent) -> bool
+        """
+        Handle a `QGraphicsScene.dropEvent`
+
+        .. versionadded:: 0.1.20
+        """
+        return False
+
 
 class NoPossibleLinksError(ValueError):
     pass
@@ -285,7 +330,7 @@ class NewLinkAction(UserInteraction):
         self.macro = None  # type: Optional[UndoCommand]
 
         # Cache viable signals of currently hovered node
-        self.__target_compatible_signals = None
+        self.__target_compatible_signals: Sequence[Tuple[OutputSignal, InputSignal]] = []
 
         self.cancelOnEsc = True
 
@@ -316,7 +361,7 @@ class NewLinkAction(UserInteraction):
                                                        self.__target_compatible_signals)
         self.tmp_anchor_point.setSignal(signal)
 
-    def create_tmp_anchor(self, item, scenePos, viableLinks=None):
+    def create_tmp_anchor(self, item, scenePos):
         # type: (items.NodeItem, QPointF) -> None
         """
         Create a new tmp anchor at the `item` (:class:`NodeItem`).
@@ -333,29 +378,33 @@ class NewLinkAction(UserInteraction):
                                         self.__target_compatible_signals)
             self.tmp_anchor_point = item.newOutputAnchor(signal)
 
-    def can_connect(self, target_item):
-        # type: (items.NodeItem) -> bool
+    def __possible_connection_signal_pairs(
+            self, target_item: items.NodeItem
+    ) -> Sequence[Tuple[OutputSignal, InputSignal]]:
         """
-        Is the connection between `self.from_item` (item where the drag
-        started) and `target_item` possible.
-
-        If possible, initialize the variables regarding the node.
+        Return possible connection signal pairs between current
+        `self.from_item` and `target_item`.
         """
         if self.from_item is None:
-            return False
+            return []
         node1 = self.scene.node_for_item(self.from_item)
         node2 = self.scene.node_for_item(target_item)
 
         if self.direction == self.FROM_SOURCE:
             links = self.scheme.propose_links(node1, node2,
                                               source_signal=self.from_signal)
-            self.__target_compatible_signals = [l[1] for l in links]
         else:
             links = self.scheme.propose_links(node2, node1,
                                               sink_signal=self.from_signal)
-            self.__target_compatible_signals = [l[0] for l in links]
+        return [(s1, s2) for s1, s2, _ in links]
 
-        return bool(links)
+    def can_connect(self, target_item):
+        # type: (items.NodeItem) -> bool
+        """
+        Is the connection between `self.from_item` (item where the drag
+        started) and `target_item` possible.
+        """
+        return bool(self.__possible_connection_signal_pairs(target_item))
 
     def set_link_target_anchor(self, anchor):
         # type: (items.AnchorPoint) -> None
@@ -472,11 +521,11 @@ class NewLinkAction(UserInteraction):
                 anchor = self.current_target_item.inputAnchorItem
             else:
                 anchor = self.current_target_item.outputAnchorItem
-            if not self.showing_incompatible_widget:
-                self.remove_tmp_anchor()
-                self.showing_incompatible_widget = True
-            else:
+            if self.showing_incompatible_widget:
                 anchor.setIncompatible(False)
+                self.showing_incompatible_widget = False
+            else:
+                self.remove_tmp_anchor()
             anchor.setHovered(False)
             anchor.setCompatibleSignals(None)
             self.current_target_item = None
@@ -493,12 +542,15 @@ class NewLinkAction(UserInteraction):
                     self.set_link_target_anchor(self.cursor_anchor_point)
             elif self.can_connect(item):
                 # Mouse is over a new item
+                links = self.__possible_connection_signal_pairs(item)
                 log.info("%r is the new target.", item)
                 if self.direction == self.FROM_SOURCE:
+                    self.__target_compatible_signals = [s2 for s1, s2 in links]
                     item.inputAnchorItem.setCompatibleSignals(
                         self.__target_compatible_signals)
                     item.inputAnchorItem.setHovered(True)
                 else:
+                    self.__target_compatible_signals = [s1 for s1, s2 in links]
                     item.outputAnchorItem.setCompatibleSignals(
                         self.__target_compatible_signals)
                     item.outputAnchorItem.setHovered(True)
@@ -524,6 +576,8 @@ class NewLinkAction(UserInteraction):
                 self.set_link_target_anchor(self.cursor_anchor_point)
                 self.current_target_item = item
         else:
+            self.showing_incompatible_widget = item is not None
+            self.__target_compatible_signals = []
             self.set_link_target_anchor(self.cursor_anchor_point)
 
         self.cursor_anchor_point.setPos(event.scenePos())
@@ -641,7 +695,7 @@ class NewLinkAction(UserInteraction):
         menu.setFilterFunc(filter)
         menu.triggerSearch()
         try:
-            action = menu.exec_(pos)
+            action = menu.exec(pos)
         finally:
             menu.setFilterFunc(None)
 
@@ -701,9 +755,11 @@ class NewLinkAction(UserInteraction):
                     show_link_dialog = True
 
                 # Check for destructive action (i.e. would the new link
-                # replace a previous link)
-                if sink.single and self.scheme.find_links(sink_node=sink_node,
-                                                          sink_channel=sink):
+                # replace a previous link) except for explicit only link
+                # candidates
+                if sink.single and w2 > 0 and \
+                        self.scheme.find_links(sink_node=sink_node,
+                                               sink_channel=sink):
                     show_link_dialog = True
 
             if show_link_dialog:
@@ -906,7 +962,7 @@ def edit_links(
     dlg.setLinks(initial_links)
 
     log.info("Executing a Link Editor Dialog.")
-    rval = dlg.exec_()
+    rval = dlg.exec()
 
     if rval == EditLinksDialog.Accepted:
         edited_links = dlg.links()
@@ -1005,7 +1061,7 @@ class NewNodeAction(UserInteraction):
 
         menu.setSortingFunc(defaultSort)
 
-        action = menu.exec_(pos, search_text)
+        action = menu.exec(pos, search_text)
         if action:
             item = action.property("item")
             desc = item.data(QtWidgetRegistry.WIDGET_DESC_ROLE)
@@ -1125,8 +1181,8 @@ class RectangleSelectionAction(UserInteraction):
                                     Qt.IntersectsItemShape,
                                     Qt.AscendingOrder)
 
-        selected = set([item for item in selected if \
-                        item.flags() & Qt.ItemIsSelectable])
+        selected = set([item for item in selected if
+                        item.flags() & QGraphicsItem.ItemIsSelectable])
 
         if self.modifiers & Qt.ControlModifier:
             for item in selected | self.last_selection | \
@@ -1219,7 +1275,7 @@ class EditNodeLinksAction(UserInteraction):
         dlg.setLinks(initial_links)
 
         log.info("Executing a Link Editor Dialog.")
-        rval = dlg.exec_()
+        rval = dlg.exec()
 
         if rval == EditLinksDialog.Accepted:
             links_spec = dlg.links()
@@ -1753,3 +1809,368 @@ class ResizeArrowAnnotation(UserInteraction):
         self.annotation = None
 
         super().end()
+
+
+class DropHandler(abc.ABC):
+    """
+    An abstract drop handler.
+
+    .. versionadded:: 0.1.20
+    """
+    @abc.abstractmethod
+    def accepts(self, document: 'SchemeEditWidget', event: 'QGraphicsSceneDragDropEvent') -> bool:
+        """
+        Returns True if a `document` can accept a drop of the data from `event`.
+        """
+        return False
+
+    @abc.abstractmethod
+    def doDrop(self, document: 'SchemeEditWidget', event: 'QGraphicsSceneDragDropEvent') -> bool:
+        """
+        Complete the drop of data from `event` onto the `document`.
+        """
+        return False
+
+
+class DropHandlerAction(abc.ABC):
+    @abc.abstractmethod
+    def actionFromDropEvent(
+            self, document: 'SchemeEditWidget', event: 'QGraphicsSceneDragDropEvent'
+    ) -> QAction:
+        """
+        Create and return an QAction representing a drop action.
+
+        This action is used to disambiguate between possible drop actions.
+
+        The action can have sub menus, however all actions in submenus **must**
+        have the `DropHandler` instance set as their `QAction.data()`.
+
+        The actions **must not** execute the actual drop from their triggered
+        slot connections. The drop will be dispatched to the `action.data()`
+        handler's `doDrop()` after that action is triggered and the menu is
+        closed.
+        """
+        raise NotImplementedError
+
+
+class NodeFromMimeDataDropHandler(DropHandler, DropHandlerAction):
+    """
+    Create a new node from dropped mime data.
+
+    Subclasses must override `canDropMimeData`, `parametersFromMimeData`,
+    and `qualifiedName`.
+
+    .. versionadded:: 0.1.20
+    """
+    @abc.abstractmethod
+    def qualifiedName(self) -> str:
+        """
+        The qualified name for the node created by this handler. The handler
+        will not be invoked if this name does not appear in the registry
+        associated with the workflow.
+        """
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def canDropMimeData(self, document: 'SchemeEditWidget', data: 'QMimeData') -> bool:
+        """
+        Can the handler create a node from the drop mime data.
+
+        Reimplement this in a subclass to check if the `data` has appropriate
+        format.
+        """
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def parametersFromMimeData(self, document: 'SchemeEditWidget', data: 'QMimeData') -> 'Dict[str, Any]':
+        """
+        Return the node parameters based from the drop mime data.
+        """
+        raise NotImplementedError
+
+    def accepts(self, document: 'SchemeEditWidget', event: 'QGraphicsSceneDragDropEvent') -> bool:
+        """Reimplemented."""
+        reg = document.registry()
+        if not reg.has_widget(self.qualifiedName()):
+            return False
+        return self.canDropMimeData(document, event.mimeData())
+
+    def nodeFromMimeData(self, document: 'SchemeEditWidget', data: 'QMimeData') -> 'Node':
+        reg = document.registry()
+        wd = reg.widget(self.qualifiedName())
+        node = document.newNodeHelper(wd)
+        parameters = self.parametersFromMimeData(document, data)
+        node.properties = parameters
+        return node
+
+    def doDrop(self, document: 'SchemeEditWidget', event: 'QGraphicsSceneDragDropEvent') -> bool:
+        """Reimplemented."""
+        reg = document.registry()
+        if not reg.has_widget(self.qualifiedName()):
+            return False
+        node = self.nodeFromMimeData(document, event.mimeData())
+        node.position = (event.scenePos().x(), event.scenePos().y())
+        activate = self.shouldActivateNode()
+        wd = document.widgetManager()
+        if activate and wd is not None:
+            def activate(node_, widget):
+                if node_ is node:
+                    try:
+                        self.activateNode(document, node, widget)
+                    finally:
+                        # self-disconnect the slot
+                        wd.widget_for_node_added.disconnect(activate)
+            wd.widget_for_node_added.connect(activate)
+        document.addNode(node)
+        if activate:
+            QApplication.postEvent(node, WorkflowEvent(WorkflowEvent.NodeActivateRequest))
+        return True
+
+    def shouldActivateNode(self) -> bool:
+        """
+        Should the new dropped node activate (open GUI controller) immediately.
+
+        If this method returns `True` then the `activateNode` method will be
+        called after the node has been added and the GUI controller created.
+
+        The default implementation returns False.
+        """
+        return False
+
+    def activateNode(self, document: 'SchemeEditWidget', node: 'Node', widget: 'QWidget') -> None:
+        """
+        Activate (open) the `node`'s GUI controller `widget` after a
+        completed drop.
+
+        Reimplement this if the node requires further configuration via the
+        GUI.
+
+        The default implementation delegates to the :class:`WidgetManager`
+        associated with the document.
+        """
+        wd = document.widgetManager()
+        if wd is not None:
+            wd.activate_widget_for_node(node, widget)
+        else:
+            widget.show()
+
+    def actionFromDropEvent(
+            self, document: 'SchemeEditWidget', event: 'QGraphicsSceneDragDropEvent'
+    ) -> QAction:
+        """Reimplemented."""
+        reg = document.registry()
+        ac = QAction(None)
+        ac.setData(self)
+        if reg is not None:
+            desc = reg.widget(self.qualifiedName())
+            ac.setText(desc.name)
+            ac.setToolTip(tooltip_helper(desc))
+            ac.setWhatsThis(whats_this_helper(desc))
+        else:
+            ac.setText(f"{self.qualifiedName()}")
+            ac.setEnabled(False)
+            ac.setVisible(False)
+        return ac
+
+
+def load_entry_point(
+        ep: EntryPoint, log: logging.Logger = None,
+) -> Tuple['EntryPoint', Any]:
+    if log is None:
+        log = logging.getLogger(__name__)
+    try:
+        value = ep.load()
+    except (ImportError, AttributeError):
+        log.exception("Could not load %s", ep)
+    except Exception:  # noqa
+        log.exception("Unexpected Error; %s will be skipped", ep)
+    else:
+        return ep, value
+
+
+def iter_load_entry_points(
+        iter: Iterable[EntryPoint], log: logging.Logger = None,
+):
+    if log is None:
+        log = logging.getLogger(__name__)
+    for ep in iter:
+        try:
+            ep, value = load_entry_point(ep, log)
+        except Exception:
+            pass
+        else:
+            yield ep, value
+
+
+class PluginDropHandler(DropHandler):
+    """
+    Delegate drop event processing to plugin drop handlers.
+
+    .. versionadded:: 0.1.20
+    """
+    #: The default entry point group
+    ENTRY_POINT = "orangecanvas.document.interactions.DropHandler"
+
+    def __init__(self, group=ENTRY_POINT, **kwargs):
+        super().__init__(**kwargs)
+        self.__group = group
+
+    def iterEntryPoints(self) -> Iterable['EntryPoint']:
+        """
+        Return an iterator over all entry points.
+        """
+        return entry_points().get(self.__group, [])
+
+    __entryPoints = None
+
+    def entryPoints(self) -> Iterable[Tuple['EntryPoint', 'DropHandler']]:
+        """
+        Return an iterator over entry points and instantiated drop handlers.
+        """
+        eps = []
+        if self.__entryPoints:
+            ep_iter = self.__entryPoints
+            store_eps = lambda ep, value: None
+        else:
+            ep_iter = self.iterEntryPoints()
+            ep_iter = iter_load_entry_points(ep_iter, log)
+            store_eps = lambda ep, value: eps.append((ep, value))
+
+        for ep, value in ep_iter:
+            if not issubclass(value, DropHandler):
+                log.error(
+                    f"{ep} yielded {type(value)}, expected a "
+                    f"{DropHandler} subtype"
+                )
+                continue
+            try:
+                handler = value()
+            except Exception:  # noqa
+                log.exception("Error in default constructor of %s", value)
+            else:
+                yield ep, handler
+                store_eps(ep, value)
+
+        self.__entryPoints = tuple(eps)
+
+    __accepts: Sequence[Tuple[EntryPoint, DropHandler]] = ()
+
+    def accepts(self, document: 'SchemeEditWidget', event: 'QGraphicsSceneDragDropEvent') -> bool:
+        """
+        Reimplemented.
+
+        Accept the event if any plugin handlers accept the event.
+        """
+        accepts = []
+        self.__accepts = ()
+        for ep, handler in self.entryPoints():
+            if handler.accepts(document, event):
+                accepts.append((ep, handler))
+        self.__accepts = tuple(accepts)
+        return bool(accepts)
+
+    def doDrop(
+            self, document: 'SchemeEditWidget', event: 'QGraphicsSceneDragDropEvent'
+    ) -> bool:
+        """
+        Reimplemented.
+
+        Dispatch the drop to the plugin handler that accepted the event.
+        In case there are multiple handlers that accepted the event, a menu
+        is used to select the handler.
+        """
+        handler: Optional[DropHandler] = None
+        if len(self.__accepts) == 1:
+            ep, handler = self.__accepts[0]
+        elif len(self.__accepts) > 1:
+            menu = QMenu(event.widget())
+            for ep_, handler_ in self.__accepts:
+                ac = action_for_handler(handler_, document, event)
+                if ac is None:
+                    ac = menu.addAction(ep_.name, )
+                else:
+                    menu.addAction(ac)
+                    ac.setParent(menu)
+                if not ac.toolTip():
+                    ac.setToolTip(f"{ep_.name} ({ep_.module_name})")
+                ac.setData(handler_)
+            action = menu.exec(event.screenPos())
+            if action is not None:
+                handler = action.data()
+        if handler is None:
+            return False
+        return handler.doDrop(document, event)
+
+
+def action_for_handler(handler: DropHandler, document, event) -> Optional[QAction]:
+    if isinstance(handler, DropHandlerAction):
+        return handler.actionFromDropEvent(document, event)
+    else:
+        return None
+
+
+class DropAction(UserInteraction):
+    """
+    A drop action on the workflow.
+    """
+    def __init__(
+            self, document, *args, dropHandlers: Sequence[DropHandler] = (),
+            **kwargs
+    ) -> None:
+        super().__init__(document, *args, **kwargs)
+        self.__designatedAction: Optional[DropHandler] = None
+        self.__dropHandlers = dropHandlers
+
+    def dropHandlers(self) -> Iterable[DropHandler]:
+        """Return an iterable over drop handlers."""
+        return iter(self.__dropHandlers)
+
+    def canHandleDrop(self, event: 'QGraphicsSceneDragDropEvent') -> bool:
+        """
+        Can this interactions handle the drop `event`.
+
+        The default implementation checks each `dropHandler` if it
+        :func:`~DropHandler.accepts` the event. The first such handler that
+        accepts is selected to be the designated handler and will receive
+        the drop (:func:`~DropHandler.doDrop`).
+        """
+        for ep in self.dropHandlers():
+            if ep.accepts(self.document, event):
+                self.__designatedAction = ep
+                return True
+        else:
+            return False
+
+    def dragEnterEvent(self, event):
+        if self.canHandleDrop(event):
+            event.acceptProposedAction()
+            return True
+        else:
+            return False
+
+    def dragMoveEvent(self, event):
+        if self.canHandleDrop(event):
+            event.acceptProposedAction()
+            return True
+        else:
+            return False
+
+    def dragLeaveEvent(self, even):
+        self.__designatedAction = None
+        self.end()
+        return False
+
+    def dropEvent(self, event):
+        if self.__designatedAction is not None:
+            try:
+                res = self.__designatedAction.doDrop(self.document, event)
+            except Exception:  # noqa
+                log.exception("")
+                res = False
+            if res:
+                event.acceptProposedAction()
+            self.end()
+            return True
+        else:
+            self.end()
+            return False
