@@ -1,3 +1,4 @@
+import enum
 import operator
 import types
 from functools import reduce
@@ -8,6 +9,10 @@ from typing import (
     SupportsInt
 )
 
+from AnyQt.QtCore import Qt
+from AnyQt.QtGui import QMouseEvent, QWheelEvent
+from AnyQt.QtWidgets import QSizePolicy
+
 from .qtcompat import toPyObject
 
 __all__ = [
@@ -27,6 +32,11 @@ __all__ = [
     "mapping_get",
     "findf",
     "set_flag",
+    "is_flag_set",
+    "qsizepolicy_is_expanding",
+    "qsizepolicy_is_shrinking",
+    "is_event_source_mouse",
+    "UNUSED",
 ]
 
 if typing.TYPE_CHECKING:
@@ -268,8 +278,66 @@ def findf(iterable, predicate, default=None):
 
 
 def set_flag(flags, mask, on=True):
-    # type: (F, SupportsInt, bool) -> F
+    # type: (F, Union[SupportsInt, enum.Flag], bool) -> F
+    if not isinstance(mask, enum.Flag):
+        mask = int(mask)
     if on:
-        return type(flags)(flags | int(mask))
+        return type(flags)(flags | mask)
     else:
-        return type(flags)(flags & ~int(mask))
+        return type(flags)(flags & ~mask)
+
+
+def enum_as_int(value: Union[int, enum.Enum]) -> int:
+    """
+    Return a `enum.Enum` value as an `int.
+
+    This is function intended for extracting underlying Qt5/6 enum
+    values specifically with PyQt6 where most Qt enums are represented
+    with `enum.Enum` and lose their numerical value.
+
+    >>> from PyQt6.QtCore import Qt
+    >>> enum_as_int(Qt.Alignment.AlignLeft)
+    1
+    """
+    if isinstance(value, enum.Enum):
+        return int(value.value)
+    else:
+        return int(value)
+
+
+def is_flag_set(flags, mask):
+    flags_i = enum_as_int(flags)
+    mask_i = enum_as_int(mask)
+    return bool(flags_i & mask_i)
+
+
+def qsizepolicy_is_expanding(policy: QSizePolicy.Policy) -> bool:
+    return is_flag_set(policy, QSizePolicy.ExpandFlag)
+
+
+def qsizepolicy_is_shrinking(policy: QSizePolicy.Policy) -> bool:
+    return is_flag_set(policy, QSizePolicy.ShrinkFlag)
+
+
+def is_event_source_mouse(event: Union[QWheelEvent, QMouseEvent]) -> bool:
+    """
+    Does th event originate from a mouse type device or from another source
+    (touchpad/screen).
+    """
+    try:
+        return event.source() != Qt.MouseEventNotSynthesized
+    except AttributeError:  # PyQt6
+        from AnyQt.QtGui import QInputDevice
+        return event.device().type() == QInputDevice.DeviceType.Mouse
+
+
+def UNUSED(*_unused_args) -> None:
+    """
+    *Mark* the function arguments as unused for a code checker
+
+    Examples
+    --------
+    >>> def foo(bar, baz):
+    ...     UNUSED(bar, baz)
+    ...     return True
+    """
